/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSIP_H
#define CONEXUSIP_H

#include <conexus/socket.h>
#include <conexus/ipaddress_interface.h>

namespace Conexus
{

  /**
   * This class serves as the base class for all IP communication classes.
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   * @ingroup Endpoint
   */
  class IP : public Socket
  {

    protected:

      IP( int domain = -1, int type = -1, int protocol = 0 );

    public:
      /** Class typedef to smart pointer */
      typedef ConexusPointer<IP> pointer;

      virtual ~IP() throw();

      virtual IPAddressInterface& generic_local_address() = 0;

      virtual IPAddressInterface& generic_remote_address() = 0;

      
      /**
       * Get the Time-To-Live (TTL) field associated with this endpoint.
       *
       * The TTL field is sent with every packet originating from this endpoint.
       *
       * The TTL field is set by the sender of the datagram, and reduced by
       * every host on the route to its destination. If the TTL field reaches
       * zero before the datagram arrives at its destination, then the datagram
       * is discarded and an ICMP error datagram (11 - Time Exceeded) is sent
       * back to the sender.
       *
       * The purpose of the TTL field is to avoid a situation in which an
       * undeliverable datagram keeps circulating on an internet system, and
       * such a system eventually becoming swamped by such “immortal” datagrams.
       *
       * In theory, time to live is measured in seconds, although every host
       * that passes the datagram must reduce the TTL by at least one unit. In
       * practice, the TTL field is reduced by one on every hop.
       */
      int ttl();

      /**
       * Set the Time-To-Live (TTL) field associated with this endpoint.
       *
       * The TTL field is sent with every packet originating from this endpoint.
       *
       * The TTL field is set by the sender of the datagram, and reduced by
       * every host on the route to its destination. If the TTL field reaches
       * zero before the datagram arrives at its destination, then the datagram
       * is discarded and an ICMP error datagram (11 - Time Exceeded) is sent
       * back to the sender.
       *
       * The purpose of the TTL field is to avoid a situation in which an
       * undeliverable datagram keeps circulating on an internet system, and
       * such a system eventually becoming swamped by such “immortal” datagrams.
       *
       * In theory, time to live is measured in seconds, although every host
       * that passes the datagram must reduce the TTL by at least one unit. In
       * practice, the TTL field is reduced by one on every hop.
       */
      void set_ttl( int t );

      /**
       * Get the Type-Of-Service (TOS) field associated with this endpoint.
       *
       * The TOS field is sent with every packet originating from this endpoint,
       * and can be used to prioritize network traffic.
       */
      TOS tos();

      /**
       * Set the Type-Of-Service (TOS) field associated with this endpoint.
       *
       * The TOS field is sent with every packet originating from this endpoint,
       * and can be used to prioritize network traffic.
       */
      void set_tos( TOS t );

  };

}

#endif
